"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.RtcEngineExInternal = void 0;
var _tsInterfaceChecker = require("ts-interface-checker");
var _AgoraMediaBase = require("../AgoraMediaBase");
var _IAgoraRtcEngineExImpl = require("../impl/IAgoraRtcEngineExImpl");
var _AgoraBaseTi = _interopRequireDefault(require("../ti/AgoraBase-ti"));
var _AgoraMediaBaseTi = _interopRequireDefault(require("../ti/AgoraMediaBase-ti"));
var _IAgoraRtcEngineTi = _interopRequireDefault(require("../ti/IAgoraRtcEngine-ti"));
var _IrisApiEngine = require("./IrisApiEngine");
var _LocalSpatialAudioEngineInternal = require("./LocalSpatialAudioEngineInternal");
var _MediaEngineInternal = require("./MediaEngineInternal");
var _MediaPlayerInternal = require("./MediaPlayerInternal");
var _MediaRecorderInternal = require("./MediaRecorderInternal");
var _MusicContentCenterInternal = require("./MusicContentCenterInternal");
function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }
const checkers = (0, _tsInterfaceChecker.createCheckers)(_AgoraBaseTi.default, _AgoraMediaBaseTi.default, _IAgoraRtcEngineTi.default);
class RtcEngineExInternal extends _IAgoraRtcEngineExImpl.IRtcEngineExImpl {
  static _event_handlers = [];
  static _direct_cdn_streaming_event_handler = [];
  static _metadata_observer = [];
  static _audio_encoded_frame_observers = [];
  static _audio_spectrum_observers = [];
  _media_engine = new _MediaEngineInternal.MediaEngineInternal();
  _music_content_center = new _MusicContentCenterInternal.MusicContentCenterInternal();
  _local_spatial_audio_engine = new _LocalSpatialAudioEngineInternal.LocalSpatialAudioEngineInternal();
  initialize(context) {
    const ret = super.initialize(context);
    _IrisApiEngine.callIrisApi.call(this, 'RtcEngine_setAppType', {
      appType: 8
    });
    return ret;
  }
  release() {
    let sync = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : false;
    this._media_engine.release();
    this._local_spatial_audio_engine.release();
    RtcEngineExInternal._event_handlers = [];
    RtcEngineExInternal._direct_cdn_streaming_event_handler = [];
    RtcEngineExInternal._metadata_observer = [];
    RtcEngineExInternal._audio_encoded_frame_observers = [];
    RtcEngineExInternal._audio_spectrum_observers = [];
    _MediaPlayerInternal.MediaPlayerInternal._source_observers.clear();
    _MediaPlayerInternal.MediaPlayerInternal._audio_frame_observers.clear();
    _MediaPlayerInternal.MediaPlayerInternal._video_frame_observers.clear();
    _MediaPlayerInternal.MediaPlayerInternal._audio_spectrum_observers.clear();
    _MediaRecorderInternal.MediaRecorderInternal._observers.clear();
    this.removeAllListeners();
    super.release(sync);
  }
  _addListenerPreCheck(eventType) {
    var _checkers$IRtcEngineE, _checkers$IDirectCdnS, _checkers$IMetadataOb, _checkers$IAudioEncod, _checkers$IAudioSpect;
    if ((_checkers$IRtcEngineE = checkers.IRtcEngineEventHandler) !== null && _checkers$IRtcEngineE !== void 0 && _checkers$IRtcEngineE.strictTest({
      [eventType]: undefined
    })) {
      if (RtcEngineExInternal._event_handlers.length === 0) {
        this.registerEventHandler({});
      }
    }
    if ((_checkers$IDirectCdnS = checkers.IDirectCdnStreamingEventHandler) !== null && _checkers$IDirectCdnS !== void 0 && _checkers$IDirectCdnS.strictTest({
      [eventType]: undefined
    })) {
      if (RtcEngineExInternal._direct_cdn_streaming_event_handler.length === 0) {
        console.error('Please call `startDirectCdnStreaming` before you want to receive event by `addListener`');
        return false;
      }
    }
    if ((_checkers$IMetadataOb = checkers.IMetadataObserver) !== null && _checkers$IMetadataOb !== void 0 && _checkers$IMetadataOb.strictTest({
      [eventType]: undefined
    })) {
      if (RtcEngineExInternal._metadata_observer.length === 0) {
        console.error('Please call `registerMediaMetadataObserver` before you want to receive event by `addListener`');
        return false;
      }
    }
    if ((_checkers$IAudioEncod = checkers.IAudioEncodedFrameObserver) !== null && _checkers$IAudioEncod !== void 0 && _checkers$IAudioEncod.strictTest({
      [eventType]: undefined
    })) {
      if (RtcEngineExInternal._audio_encoded_frame_observers.length === 0) {
        console.error('Please call `registerAudioEncodedFrameObserver` before you want to receive event by `addListener`');
        return false;
      }
    }
    if ((_checkers$IAudioSpect = checkers.IAudioSpectrumObserver) !== null && _checkers$IAudioSpect !== void 0 && _checkers$IAudioSpect.strictTest({
      [eventType]: undefined
    })) {
      if (RtcEngineExInternal._audio_spectrum_observers.length === 0) {
        this.registerAudioSpectrumObserver({});
      }
    }
    return true;
  }
  addListener(eventType, listener) {
    this._addListenerPreCheck(eventType);
    const callback = (eventProcessor, data) => {
      if (eventProcessor.type(data) !== _IrisApiEngine.EVENT_TYPE.IRtcEngine) {
        return;
      }
      eventProcessor.func.map(it => {
        it({
          [eventType]: listener
        }, eventType, data);
      });
    };
    // @ts-ignore
    listener.agoraCallback = callback;
    _IrisApiEngine.DeviceEventEmitter.addListener(eventType, callback);
  }
  removeListener(eventType, listener) {
    _IrisApiEngine.DeviceEventEmitter.removeListener(eventType,
    // @ts-ignore
    (listener === null || listener === void 0 ? void 0 : listener.agoraCallback) ?? listener);
  }
  removeAllListeners(eventType) {
    _IrisApiEngine.DeviceEventEmitter.removeAllListeners(eventType);
  }
  getVersion() {
    const apiType = 'RtcEngine_getVersion';
    const jsonParams = {};
    const jsonResults = _IrisApiEngine.callIrisApi.call(this, apiType, jsonParams);
    return {
      build: jsonResults.build,
      version: jsonResults.result
    };
  }
  registerEventHandler(eventHandler) {
    if (!RtcEngineExInternal._event_handlers.find(value => value === eventHandler)) {
      RtcEngineExInternal._event_handlers.push(eventHandler);
    }
    return super.registerEventHandler(eventHandler);
  }
  unregisterEventHandler(eventHandler) {
    RtcEngineExInternal._event_handlers = RtcEngineExInternal._event_handlers.filter(value => value !== eventHandler);
    return super.unregisterEventHandler(eventHandler);
  }
  createMediaPlayer() {
    // @ts-ignore
    const mediaPlayerId = super.createMediaPlayer();
    return new _MediaPlayerInternal.MediaPlayerInternal(mediaPlayerId);
  }
  destroyMediaPlayer(mediaPlayer) {
    var _mediaPlayer$release;
    const ret = super.destroyMediaPlayer(mediaPlayer);
    (_mediaPlayer$release = mediaPlayer.release) === null || _mediaPlayer$release === void 0 ? void 0 : _mediaPlayer$release.call(mediaPlayer);
    return ret;
  }
  createMediaRecorder(info) {
    // @ts-ignore
    const nativeHandle = super.createMediaRecorder(info);
    return new _MediaRecorderInternal.MediaRecorderInternal(nativeHandle);
  }
  destroyMediaRecorder(mediaRecorder) {
    var _mediaRecorder$releas;
    const ret = super.destroyMediaRecorder(mediaRecorder);
    (_mediaRecorder$releas = mediaRecorder.release) === null || _mediaRecorder$releas === void 0 ? void 0 : _mediaRecorder$releas.call(mediaRecorder);
    return ret;
  }
  startDirectCdnStreaming(eventHandler, publishUrl, options) {
    if (!RtcEngineExInternal._direct_cdn_streaming_event_handler.find(value => value === eventHandler)) {
      RtcEngineExInternal._direct_cdn_streaming_event_handler.push(eventHandler);
    }
    return super.startDirectCdnStreaming(eventHandler, publishUrl, options);
  }
  registerMediaMetadataObserver(observer, type) {
    if (!RtcEngineExInternal._metadata_observer.find(value => value === observer)) {
      RtcEngineExInternal._metadata_observer.push(observer);
    }
    return super.registerMediaMetadataObserver(observer, type);
  }
  unregisterMediaMetadataObserver(observer, type) {
    RtcEngineExInternal._metadata_observer = RtcEngineExInternal._metadata_observer.filter(value => value !== observer);
    return super.unregisterMediaMetadataObserver(observer, type);
  }
  getApiTypeFromJoinChannel(token, channelId, uid, options) {
    return 'RtcEngine_joinChannel2';
  }
  getApiTypeFromLeaveChannel(options) {
    return options === undefined ? 'RtcEngine_leaveChannel' : 'RtcEngine_leaveChannel2';
  }
  getApiTypeFromSetClientRole(role, options) {
    return options === undefined ? 'RtcEngine_setClientRole' : 'RtcEngine_setClientRole2';
  }
  getApiTypeFromStartEchoTest(config) {
    return 'RtcEngine_startEchoTest3';
  }
  getApiTypeFromStartPreview() {
    let sourceType = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _AgoraMediaBase.VideoSourceType.VideoSourceCameraPrimary;
    return 'RtcEngine_startPreview2';
  }
  getApiTypeFromStopPreview() {
    let sourceType = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : _AgoraMediaBase.VideoSourceType.VideoSourceCameraPrimary;
    return 'RtcEngine_stopPreview2';
  }
  getApiTypeFromStartAudioRecording(config) {
    return 'RtcEngine_startAudioRecording3';
  }
  getApiTypeFromStartAudioMixing(filePath, loopback, cycle) {
    let startPos = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 0;
    return 'RtcEngine_startAudioMixing2';
  }
  getApiTypeFromEnableDualStreamMode(enabled, streamConfig) {
    return streamConfig === undefined ? 'RtcEngine_enableDualStreamMode' : 'RtcEngine_enableDualStreamMode2';
  }
  getApiTypeFromSetDualStreamMode(mode, streamConfig) {
    return streamConfig === undefined ? 'RtcEngine_setDualStreamMode' : 'RtcEngine_setDualStreamMode2';
  }
  getApiTypeFromLeaveChannelEx(connection, options) {
    return 'RtcEngineEx_leaveChannelEx2';
  }
  getApiTypeFromCreateDataStream(config) {
    return 'RtcEngine_createDataStream2';
  }
  getApiTypeFromAddVideoWatermark(watermarkUrl, options) {
    return 'RtcEngine_addVideoWatermark2';
  }
  getApiTypeFromJoinChannelWithUserAccount(token, channelId, userAccount, options) {
    return options === undefined ? 'RtcEngine_joinChannelWithUserAccount' : 'RtcEngine_joinChannelWithUserAccount2';
  }
  getApiTypeFromCreateDataStreamEx(config, connection) {
    return 'RtcEngineEx_createDataStreamEx2';
  }
  getApiTypeFromPreloadChannelWithUserAccount(token, channelId, userAccount) {
    return 'RtcEngine_preloadChannel2';
  }
  getAudioDeviceManager() {
    throw 'Not support';
  }
  getVideoDeviceManager() {
    throw 'Not support';
  }
  getMediaEngine() {
    return this._media_engine;
  }
  getMusicContentCenter() {
    return this._music_content_center;
  }
  getLocalSpatialAudioEngine() {
    return this._local_spatial_audio_engine;
  }
  registerAudioEncodedFrameObserver(config, observer) {
    if (!RtcEngineExInternal._audio_encoded_frame_observers.find(value => value === observer)) {
      RtcEngineExInternal._audio_encoded_frame_observers.push(observer);
    }
    return super.registerAudioEncodedFrameObserver(config, observer);
  }
  unregisterAudioEncodedFrameObserver(observer) {
    RtcEngineExInternal._audio_encoded_frame_observers = RtcEngineExInternal._audio_encoded_frame_observers.filter(value => value !== observer);
    return super.unregisterAudioEncodedFrameObserver(observer);
  }
  registerAudioSpectrumObserver(observer) {
    if (!RtcEngineExInternal._audio_spectrum_observers.find(value => value === observer)) {
      RtcEngineExInternal._audio_spectrum_observers.push(observer);
    }
    return super.registerAudioSpectrumObserver(observer);
  }
  unregisterAudioSpectrumObserver(observer) {
    RtcEngineExInternal._audio_spectrum_observers = RtcEngineExInternal._audio_spectrum_observers.filter(value => value !== observer);
    return super.unregisterAudioSpectrumObserver(observer);
  }
}
exports.RtcEngineExInternal = RtcEngineExInternal;
//# sourceMappingURL=RtcEngineExInternal.js.map